#include <Arduino.h>
#include <Wire.h>

#include "OLED_SSD1311.h"
OLED oled(0x3C);                   // I2C Address

#include <GSMSMS.h>
GSMSMS gsmsms(46,1);               // MeGGaBoard 3G SIM5320 Power Control is on Pin 46

char *DeviceBanner="OLEDCommunicator";
char *FW="OLEDComm FW v1.1";
char *NoAgentMsg="No Agent";

#define NUMSIZE 20
char Caller[NUMSIZE];
#define BUFFSIZE 512
char Buffer[BUFFSIZE];

char Line[17];

int AgentConnected;
unsigned long AgentTimeout=60000;
unsigned long AgentLastSeen;
int SMSPending;

#define NUMCMDS 7
char *CmdArray[NUMCMDS] = { 
  "SMS_Send",
  "Display",
  "Clear",
  "SMS_Check",
  "Hello",
  "Bye", 
  "Quit"
};

void setup() {

  Serial.begin(9600);

  oled.Init();
  oled.Clear();

  oled.Display(DeviceBanner, FW, 1);

  delay(2000);

  oled.DisplayLine(2,"Starting GSM...");
  gsmsms.Init(9600);
  gsmsms.SetParams();
  gsmsms.DeleteAllSMS();
  gsmsms.SendCmd("AT+CNMI=2,1");              // Enable Unsolicited SMS Notification
  gsmsms.SendCmd("AT+AUTOCSQ=1,1");           // Enable Unsolicited CSQ Notification (whenever it changes)
  oled.DisplayLine(2,"GSM Initialized");
  delay(2000);

  if(!gsmsms.SIMInserted()) {
    oled.DisplayLine(2,"SIM NOT Inserted");
    delay(2000);
  }

  oled.DisplayLine(2, NoAgentMsg);
  
  AgentConnected=0;
  AgentLastSeen=0;
}

void loop() {
int scode;

  scode=0;
  if(CheckSerial(Buffer, BUFFSIZE, 10)) {   // Anything in the Serial Buffer?
    scode=ProcessSerial(Buffer);            // Process Serial Command   
  }
  CheckAgent(scode);
  
  if(CheckGSM(Buffer, BUFFSIZE, 10)) {      // Anything in the GSM Buffer?
    if(strstr(Buffer,"+CMTI:")) {           // SMSPending
      SMSPending++;
    }
  }
}

int CheckSerial(char *buff, int buffmax, int mstimeout) {
int c;
int i;
unsigned long LastRead;

  if(!Serial.available()) return 0;

  buff[0]='\0';
  do {
    if(Serial.available()) {
      c = Serial.read();
      i = strlen(buff); 
      if (i < (buffmax - 1)) {
        buff[i] = c;  
        buff[i+1] = '\0';
      }
      LastRead = millis();
    }
  } while ((millis() - LastRead) < mstimeout);

  return strlen(buff);
}

int CheckSerialOld(char *buff, int buffmax, int msdelay) {
int c;
int i;

  if(!Serial.available()) return 0;

  buff[0]='\0';
  while(Serial.available()) {
    c = Serial.read();
    i = strlen(buff); 
    if (i < (buffmax - 1)) {
      buff[i] = c;  
      buff[i+1] = '\0';
    }
    if(!Serial.available()) delay(msdelay);
  }

  return strlen(buff);
}

int CheckGSM(char *buff, int buffmax, int mstimeout) {
int c;
int i;
unsigned long LastRead;

  if(!gsmsms.Available()) return 0;

  buff[0]='\0';
  do {
    if(gsmsms.Available()) {
      c = gsmsms.Read();
      i = strlen(buff); 
      if (i < (buffmax - 1)) {
        buff[i] = c;  
        buff[i+1] = '\0';
      }
      LastRead = millis();
    }
  } while ((millis() - LastRead) < mstimeout);

  return strlen(buff);
}

int CheckGSMOld(char *buff, int buffmax, int msdelay) {
int c;
int i;

  if(!gsmsms.Available()) return 0;

  buff[0]='\0';
  while(gsmsms.Available()) {
    c = gsmsms.Read();
    i = strlen(buff); 
    if (i < (buffmax - 1)) {
      buff[i] = c;  
      buff[i+1] = '\0';
    }
    if(!gsmsms.Available()) delay(msdelay);
  }

  return 1;
}

int ProcessSerial(char *Buffer) {
char *p1;
char *p2;
char *p3;
int CmdID;

  CmdID=GetCommandID(Buffer, CmdArray, NUMCMDS);
  switch (CmdID) {

    case 1:     // SMS_SEND
      p1 = strchr(Buffer,'\n');
      if(p1 != NULL) {
        p1++;
        p2=strchr(p1,'\n');
        if(p2 != NULL) {
          *p2='\0';
          p2++;
          gsmsms.SendSMS(p1, p2);
          break;       
        }
      }
      Serial.print("Error: Bad Command Format");
      return 2; 

    case 2:     // DISPLAY
      p1 = strchr(Buffer,'\n');
      if(p1 != NULL) {
        p1++;
        p2=strchr(p1,'\n');
        if(p2 != NULL) {
          *p2='\0';
          p2++;
          p3=strchr(p2,'\n');
          if(p3 != NULL) *p3='\0';
          oled.Display(p1, p2);
          break;    
        }
      }
      Serial.print("Error: Bad Command Format");
      return 2; 
      
    case 3:     // CLEAR
      oled.Clear();
      break;

    case 4:     // SMS_CHECK
      if(!SMSPending) Serial.print("None");
      else if(!CheckSMS()) {
        SMSPending=0;
        Serial.print("None");
      }
      return 1;

    case 5:     // HELLO
      Serial.print(FW);
      return 1;

    case 6:     // BYE
      AgentUpdate(0);
      break;

    case 7:     // QUIT
      Serial.print("OK");
      AgentConnected=0;
      return 0;

    default:
      Serial.print("Error: Invalid Command");
      return 2; 
  }

  Serial.print("OK");
  return 1;
}

int GetCommandID(char *astring, char *Array[], int NumCmds) {
int i;
 
  for (i=0; i<NumCmds; i++) {
    if(!strncasecmp(astring,Array[i],strlen(Array[i]))) return i+1;
  }
  return 0;
}

void CheckAgent(int status) {

  if(status) {
    if(AgentConnected) AgentLastSeen=millis();
    else AgentUpdate(1);
  } else {
    if(AgentConnected) {
      if((millis()-AgentLastSeen) > AgentTimeout) AgentUpdate(0);    
    }
  }

  return;
}

void AgentUpdate(int status) {
  switch (status) {
    case 0:
      AgentConnected=0;
      oled.Display(DeviceBanner, NoAgentMsg, 1);
      break;
    case 1:
      AgentConnected=1;
      AgentLastSeen=millis();
      break;
  }
}

int CheckSMS()
{  
  if(gsmsms.GetSMS(Caller, Buffer, NUMSIZE, BUFFSIZE) == 0) return 0;

  Serial.print("SMS_RCV\n");
  Serial.print(Caller);
  Serial.print("\n");
  Serial.print(Buffer);

  return 1;
}
